// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-expect-error
import loaderUtils from 'next/dist/compiled/loader-utils3';
import {
  fetchCSSFromGoogleFonts,
  getFontAxes,
  getUrl,
  validateData,
} from '@next/font/dist/google/utils';

import type { LoaderOptions } from '../types';

const cssCache = new Map<string, Promise<string>>();

export async function getFontFaceDeclarations(options: LoaderOptions) {
  const { fontFamily, weights, styles, selectedVariableAxes, display, variable } = validateData(
    options.fontFamily,
    [options.props],
    null
  );

  const fontAxes = getFontAxes(fontFamily, weights, styles, selectedVariableAxes);
  const url = getUrl(fontFamily, fontAxes, display);

  try {
    const hasCachedCSS = cssCache.has(url);
    const fontFaceCSS = hasCachedCSS
      ? cssCache.get(url)
      : await fetchCSSFromGoogleFonts(url, fontFamily).catch(() => null);
    if (!hasCachedCSS) {
      cssCache.set(url, fontFaceCSS);
    } else {
      cssCache.delete(url);
    }
    if (fontFaceCSS === null) {
      throw Error(`Failed to fetch \`${fontFamily}\` from Google Fonts.`);
    }

    return {
      id: loaderUtils.getHashDigest(url, 'md5', 'hex', 6),
      fontFamily,
      fontFaceCSS,
      weights,
      styles,
      variable,
    };
  } catch (error) {
    throw new Error("Google Fonts couldn't be loaded.");
  }
}
